import { Codec, CodecSelector, defaultCodec } from './codec';
import { RPCMessage } from '../core/type';
import { gzip, ungzip } from 'pako';

export class GZipCodec implements Codec {
  public encode(message: string): RPCMessage {
    return <ArrayBuffer>gzip(message).buffer;
  }
  public decode(message: RPCMessage): string {
    return ungzip(Buffer.from(<ArrayBuffer>message), {to: 'string'});
  }
}

export const gZipCodec = new GZipCodec();

export class CompressionCodecSelector implements CodecSelector {
  private readonly threshold: number;

  public constructor(threshold: number) {
    this.threshold = threshold;
  }

  public encode(message: string): RPCMessage {
    if (this.threshold < Buffer.byteLength(message)) {
      return gZipCodec.encode(message);
    } else {
      return defaultCodec.encode(message);
    }
  }

  public decode(message: RPCMessage): string {
    if (message instanceof ArrayBuffer) {
      return gZipCodec.decode(message);
    } else {
      return defaultCodec.decode(message);
    }
  }
}
